# -*- coding: utf-8 -*-
#  nop.py
#
# Copyright 2011 Hiroshi Miura <miurahr@linux.com>

class NOP(object):
    def __init__(self):
        pass

    def isRegion(self, char):
        return False

    def convert(self, text):
        return ("", 0)




# -*- coding: utf-8 -*-
#  h2a.py
#
# Copyright 2011,2014 Hiroshi Miura <miurahr@linux.com>
#
# Original copyright:
# * KAKASI (Kanji Kana Simple inversion program)
# * $Id: jj2.c,v 1.7 2001-04-12 05:57:34 rug Exp $
# * Copyright (C) 1992
# * Hironobu Takahashi (takahasi@tiny.or.jp)
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either versions 2, or (at your option)
# * any later version.
# *
# * This program is distributed in the hope that it will be useful
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with KAKASI, see the file COPYING.  If not, write to the Free
# * Software Foundation Inc., 59 Temple Place - Suite 330, Boston, MA
# * 02111-1307, USA.
# */

from jisyo import jisyo

class H2a (object):

    H2a_table = {}

    # default Hepburn convert method
    def __init__(self, method="Hepburn"):
        if method == "Hepburn":
            from h2ah import H2ah
            self.__class__ = H2ah
        else:
            from h2ar import H2ar
            self.__class__ = H2ar

    def isRegion(self, char):
        return ( 0x3040 < ord(char) and ord(char) < 0x3094)

    def convert(self, text):
        Hstr = ""
        max_len = -1
        r = min(4, len(text)+1)
        for x in xrange(r):
            if text[:x] in self.H2a_table:
                if max_len < x:
                    max_len = x
                    Hstr = self.H2a_table[text[:x]]
        return (Hstr, max_len)




# -*- coding: utf-8 -*-
#  j2a.py
#
# Copyright 2011 Hiroshi Miura <miurahr@linux.com>
#
#  Original Copyright:
# * KAKASI (Kanji Kana Simple inversion program)
# * $Id: jj2.c,v 1.7 2001-04-12 05:57:34 rug Exp $
# * Copyright (C) 1992
# * Hironobu Takahashi (takahasi@tiny.or.jp)
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either versions 2, or (at your option)
# * any later version.
# *
# * This program is distributed in the hope that it will be useful
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with KAKASI, see the file COPYING.  If not, write to the Free
# * Software Foundation Inc., 59 Temple Place - Suite 330, Boston, MA
# * 02111-1307, USA.
# */

from jisyo import jisyo
import re

class J2a (object):

    _jconv = None
    _hconv = None

    def __init__(self, method="Hepburn"):
        from j2h import J2H
        from h2a import H2a
        self._jconv = J2H()
        self._hconv = H2a(method)

    def isRegion(self, char):
        return self._jconv.isRegion(char)

    def convert(self, text):
        if not self._jconv.isRegion(text[0]):
            return ("", 0)
            
        (t, l) = self._jconv.convert(text)
        if l <= 0:
            return ("", 0)

        m = 0
        otext = ""

        while True: 
            if m >= len(t):
                break
            (s, n) = self._hconv.convert(t[m:])
            if n <= 0:
                m = m + 1
            else:
                m = m + n
                otext = otext+s

        return (otext, l) 




# -*- coding: utf-8 -*-
#  kakasi.py
#
# Copyright 2011,2014 Hiroshi Miura <miurahr@linux.com>
#
#  Original Copyright:
# * KAKASI (Kanji Kana Simple inversion program)
# * $Id: jj2.c,v 1.7 2001-04-12 05:57:34 rug Exp $
# * Copyright (C) 1992
# * Hironobu Takahashi (takahasi@tiny.or.jp)
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either versions 2, or (at your option)
# * any later version.
# *
# * This program is distributed in the hope that it will be useful
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with KAKASI, see the file COPYING.  If not, write to the Free
# * Software Foundation Inc., 59 Temple Place - Suite 330, Boston, MA
# * 02111-1307, USA.
# */

import re
import sys, os

class kakasi(object):

#instances
    _jconv = None
    _hconv = None
    _kconv = None
    _aconv = None

#mode flags
    _flag = {"W":False, "C":True, "c":True}
    _mode = {"J":"a", "H":"a", "K":"a", "a":None, "r":"Hepburn"}
    _values = ["a", "h", "k"]
    _option = {"r":"Hepburn"}
    _optvals = {"r":["Hepburn", "Kunrei"]}

#variables
    _separator = ''

    def __init__(self):
        pass

#fixme: value chck
    def setMode(self, fr, to):
        if fr in self._mode:
            if to in self._values:
                self._mode[fr] = to
        if fr in self._flag:
            if to in [True,False]:
                self._flag[fr] = to
        if fr in self._option:
            if to in self._optvals[fr]:
                self._option[fr] = to

    def getConverter(self):

        if self._mode["H"] == "a":
            from h2a import H2a
            self._hconv = H2a(method = self._option["r"]) 
        else:
            from nop import NOP
            self._hconv = NOP()

        if self._mode["K"] == "a":
            from k2a import K2a
            self._kconv = K2a(method = self._option["r"])
        else:
            from nop import NOP
            self._kconv = NOP()

        if self._mode["J"] == "a":
            from j2a import J2a
            self._jconv = J2a(method = self._option["r"])
            if self._flag["C"]:
                self._separator = ' '
            else:
                self._separator = ''

        from nop import NOP
        if self._mode["a"] == None:
            self._aconv = NOP()
        else:
            self._aconv = NOP()

        return self

    def do(self, text):

        otext =  ''
        i = 0
        while True:
            if i >= len(text):
                break

            if self._jconv.isRegion(text[i]):
                (t, l) = self._jconv.convert(text[i:])
                if l <= 0:
                    i = i + 1
                    break
                i = i + l
                if self._flag["c"]:
                    t = t.capitalize()
                if i >= len(text):
                    otext = otext + t
                else:
                    otext = otext + t + self._separator

            elif self._hconv.isRegion(text[i]):
                tmptext = ''
                while True: # eat mode
                    (t, l) = self._hconv.convert(text[i:])
                    tmptext = tmptext+t
                    i = i + l
                    if i >= len(text):
                        otext = otext + tmptext
                        break
                    elif not self._hconv.isRegion(text[i]):
                        otext = otext + tmptext + self._separator
                        break

            elif self._kconv.isRegion(text[i]):
                tmptext = ''
                while True: # eat mode
                    (t, l) = self._kconv.convert(text[i:])
                    tmptext = tmptext+t
                    i = i + l
                    if i >= len(text):
                        otext = otext + tmptext
                        break
                    elif not self._kconv.isRigion(text[i]):
                        otext = otext + tmptext + self._separator
                        break

            else:
                otext  = otext + text[i]
                i += 1

        return otext




# -*- coding: utf-8 -*-
#  jisyo.py
#
# Copyright 2011 Hiroshi Miura <miurahr@linux.com>
from cPickle import load
import anydbm,marshal
from zlib import decompress
import os

class jisyo (object):
    kanwadict = None
    itaijidict = None
    jisyo_table = {}

    def __init__(self):
        if self.kanwadict is None:
            dictpath = os.path.join('pykakasi','kanwadict2.db')
            self.kanwadict = anydbm.open(dictpath,'r')
        if self.itaijidict is  None:
            itaijipath = os.path.join('pykakasi','itaijidict2.pickle')
            itaiji_pkl = open(itaijipath, 'rb')
            self.itaijidict = load(itaiji_pkl)

    def load_jisyo(self, char):
        try:#python2
            key = "%04x"%ord(unicode(char))
        except:#python3
            key = "%04x"%ord(char)

        try: #already exist?
            table = self.jisyo_table[key]
        except:
            try:
                table = self.jisyo_table[key]  = marshal.loads(decompress(self.kanwadict[key]))
            except:
                return None
        return table




#!/usr/bin/python
# derivered from unihandecode setup.py

from setuptools import Command,setup

import unittest
import os
import genkanwadict

UNITTESTS = [
        "tests", 
    ]

class TestCommand(Command):
    user_options = [ ]

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        suite = unittest.TestSuite()

        suite.addTests( 
            unittest.defaultTestLoader.loadTestsFromNames( 
                                UNITTESTS ) )

        result = unittest.TextTestRunner(verbosity=3).run(suite)

class GenKanwa(Command):
    user_options = [ ]

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        src = os.path.join('data','kakasidict.utf8')        
        dst = os.path.join('pykakasi','kanwadict2.db')
        try:
            os.unlink(dst)
        except:
            pass
        kanwa = genkanwadict.mkkanwa()
        kanwa.run(src, dst)
        src = os.path.join('data','itaijidict.utf8')
        dst = os.path.join('pykakasi','itaijidict2.pickle')
        try:
            os.unlink(dst)
        except:
            pass
        kanwa.mkitaiji(src, dst)

setup(name='pykakasi',
      version='0.01',
      description='Python implementation of kakasi - kana kanji simple inversion library',
      url='http://github.com/miurahr/pykakasi',
      license='GPLv3',
      long_description="",
      author='Hioshi Miura',
      author_email='miurahr@linux.com',
      packages = [ 'pykakasi' ],
      provides = [ 'pykakasi' ],
      include_package_data = True,
      cmdclass = { 'test': TestCommand, 'genkanwa':GenKanwa }
)



# -*- coding: utf-8 -*-
import unittest
import pykakasi

class TestPyKakasi(unittest.TestCase):

    def test_J2H(self):

        TESTS = [
            (u"構成",         (u"こうせい",2)),
            (u"好き",          (u"すき",2)),
            (u"大きい",       (u"おおき",2)),
      ]

        I_TEST = [
            (u"菟", u"兎"),
            (u"菟集", u"兎集"),
            (u"熙", u"煕"),
        ]

        j = pykakasi.J2H()
        for case, result in TESTS:
            self.failUnlessEqual(j.convert(case), result)
        for case, result in I_TEST:
            self.failUnlessEqual(j.itaiji_conv(case), result)

    def test_H2a(self):

        TESTS = [
            (u"かんたん",   ("ka", 1)),
            (u"にゃ", ("nya",2)),
            (u"っき", ("kki",2)),
            (u"っふぁ", ("ffa", 3)),
            (u"しつもん",   ("shi",1)),
            (u"ちがい", ("chi",1)),
        ]

        h = pykakasi.H2a()
        for case, result in TESTS:
            self.failUnlessEqual(h.convert(case), result)

    def test_K2a(self):

        TESTS = [
            (u"カンタン",   ("ka", 1)),
            (u"ニャ", ("nya",2)),
            (u"ッキ", ("kki",2)),
            (u"ッファ", ("ffa", 3)),
            (u"シツモン",   ("shi", 1)),
            (u"チガイ",  ("chi", 1)),
            (u"ジ", ("ji",1)),
        ]

        h = pykakasi.K2a()
        for case, result in TESTS:
            self.failUnlessEqual(h.convert(case), result)

    def test_H2a_kunrei(self):

        TESTS = [
            (u"しつもん",   ("si",1)),
            (u"ちがい", ("ti",1)),
            (u"きゃ", ("kya", 2)), (u"きゅ", ("kyu", 2)), (u"きょ", ("kyo", 2)),
            (u"しゃ", ("sya", 2)), (u"しゅ", ("syu", 2)), (u"しょ", ("syo", 2)),
            (u"ちゃ", ("tya", 2)), (u"ちゅ", ("tyu", 2)), (u"ちょ", ("tyo", 2)),
            (u"にゃ", ("nya", 2)), (u"にゅ", ("nyu", 2)), (u"にょ", ("nyo", 2)),
            (u"りゃ", ("rya", 2)), (u"りゅ", ("ryu", 2)), (u"りょ", ("ryo", 2)),
            (u"ざ", ("za", 1)), (u"じ", ("zi", 1)), (u"ず", ("zu", 1)),
            (u"ぜ", ("ze", 1)), (u"ぞ", ("zo", 1)),
            (u"だ", ("da", 1)), (u"ぢ", ("zi", 1)), (u"づ", ("zu", 1)),
            (u"で", ("de", 1)), (u"ど", ("do", 1)),
            (u"た", ("ta", 1)), (u"ち", ("ti", 1)), (u"つ", ("tu", 1)),
            (u"て", ("te", 1)), (u"と", ("to", 1))
        ]

        h = pykakasi.H2a(method="Kunrei")
        for case, result in TESTS:
            self.failUnlessEqual(h.convert(case), result)

    def test_K2a_kunrei(self):

        TESTS = [
            (u"シツモン",   ("si", 1)),
            (U"チガイ", ("ti", 1)),
            (u"ジ", ("zi",1)),
            (u"ファジー", ("fa", 2)),
            (u"ジー", ("zi", 1)),
            (u"ウォークマン", ("u", 1)),
            (u"キャ", ("kya", 2)), (u"キュ", ("kyu", 2)), (u"キョ", ("kyo", 2)),
            (u"シャ", ("sya", 2)), (u"シュ", ("syu", 2)), (u"ショ", ("syo", 2)),
            (u"チャ", ("tya", 2)), (u"チュ", ("tyu", 2)), (u"チョ", ("tyo", 2)),
            (u"ニャ", ("nya", 2)), (u"ニュ", ("nyu", 2)), (u"ニョ", ("nyo", 2)),
            (u"リャ", ("rya", 2)), (u"リュ", ("ryu", 2)), (u"リョ", ("ryo", 2)),
            (u"ザ", ("za", 1)), (u"ジ", ("zi", 1)), (u"ズ", ("zu", 1)),
            (u"ゼ", ("ze", 1)), (u"ゾ", ("zo", 1)),
            (u"ダ", ("da", 1)), (u"ヂ", ("zi", 1)), (u"ヅ", ("zu", 1)),
            (u"デ", ("de", 1)), (u"ド", ("do", 1)),
            (u"タ", ("ta", 1)), (u"チ", ("ti", 1)), (u"ツ", ("tu", 1)),
            (u"テ", ("te", 1)), (u"ト", ("to", 1))
        ]

        h = pykakasi.K2a(method="Kunrei")
        for case, result in TESTS:
            self.failUnlessEqual(h.convert(case), result)

    def test_kakasi(self):

        TESTS = [
            (u"構成",         "Kousei"),
            (u"好き",          "Suki"),
            (u"大きい",       "Ooki i"),
            (u"かんたん",  "kantan"),
            (u"にゃ",          "nya"),
            (u"っき",           "kki"),
            (u"っふぁ",        "ffa"),
            (u"漢字とひらがな交じり文", "Kanji tohiragana Majiri Bun"),
            (u"Alphabet 123 and 漢字", "Alphabet 123 and Kanji"),
            (u"日経新聞", "Nikkeishinbun"),
        ]

        kakasi = pykakasi.kakasi()
        kakasi.setMode("H","a")
        kakasi.setMode("K","a")
        kakasi.setMode("J","a")
        kakasi.setMode("r","Hepburn")
        converter  = kakasi.getConverter()
        for case, result in TESTS:
            self.failUnlessEqual(converter.do(case), result)


